"""
Controller package for Task Resource Manager.

This package contains the controller classes that coordinate
the model and view components.
"""

from .task_manager import TaskResourceManager

__all__ = ['TaskResourceManager']
