"""
Color definitions for Task Resource Manager.
Contains a list of web color names that can be used for tasks.
"""

# Dictionary of web color names and their hex values
WEB_COLORS = {
    "AliceBlue": "#F0F8FF",
    "Azure": "#F0FFFF",
    "Beige": "#F5F5DC",
    "Bisque": "#FFE4C4",
    "BlanchedAlmond": "#FFEBCD",
    "Blue": "#0000FF",
    "CadetBlue": "#5F9EA0",
    "Coral": "#FF7F50",
    "CornflowerBlue": "#6495ED",
    "Cyan": "#00FFFF",  # Default task color
    "Gold": "#FFD700",
    "GoldenRod": "#DAA520",
    "Green": "#008000",
    "GreenYellow": "#ADFF2F",
    "HotPink": "#FF69B4",
    "Khaki": "#F0E68C",
    "Lavender": "#E6E6FA",
    "LemonChiffon": "#FFFACD",
    "LightBlue": "#ADD8E6",
    "LightCyan": "#E0FFFF",
    "LightGreen": "#90EE90",
    "LightPink": "#FFB6C1",
    "LightSalmon": "#FFA07A",
    "LightSkyBlue": "#87CEFA",
    "LightYellow": "#FFFFE0",
    "Lime": "#00FF00",
    "MediumAquaMarine": "#66CDAA",
    "MediumOrchid": "#BA55D3",
    "MistyRose": "#FFE4E1",
    "Orange": "#FFA500",
    "PaleGreen": "#98FB98",
    "PaleTurquoise": "#AFEEEE",
    "PeachPuff": "#FFDAB9",
    "Pink": "#FFC0CB",
    "Plum": "#DDA0DD",
    "PowderBlue": "#B0E0E6",
    "Salmon": "#FA8072",
    "SkyBlue": "#87CEEB",
    "Thistle": "#D8BFD8",
    "Violet": "#EE82EE",
    "Yellow": "#FFFF00",
    "YellowGreen": "#9ACD32",
}

# Default color for tasks
DEFAULT_TASK_COLOR = "Cyan"

# List of color names for menus
COLOR_NAMES = sorted(WEB_COLORS.keys())
