"""
Model package for Task Resource Manager.

This package contains the data model classes for the application.
"""

from .task_resource_model import TaskResourceModel

__all__ = ['TaskResourceModel']
